%==================================================================
%  
%  TRACEO3D: LMA CNRS Experimental Data
%  Faro, Ter Jun 13 12:38:29 WEST 2017
%  Written by Tordar 
%  
%==================================================================

clear all, close all

system('rm -rf rco.mat');

disp('LMA CNRS Tank Experiment:') 

case_title = 'LMA_CNRS_Tank_Experiment';

%==================================================================
%  
%  Define source data:
%  
%==================================================================

freq = 150; ray_step = 1;

slope = 4.55;
k = tan( slope*pi/180 ); z0 = 44.4;
x1 = -z0/k;
x2 =   -x1;
z1 = k*x1 + z0;
z2 = k*x2 + z0;
zmax = max([z1 z2]);

zs = 8.3;

xs = [0 0 zs];

thetamax = 15;
nthetas = 11;
thetas = linspace(-thetamax,thetamax,nthetas);

dphi = 20;
phi = [90:1:90+dphi]; nphi = length( phi );

source_data.ds       = ray_step;
source_data.position = xs;
source_data.f	     = freq;
source_data.thetas   = thetas;
source_data.phi      = phi;
source_data.nthetas   = nthetas;
source_data.nphi      = nphi;
source_data.xbox     = [-400 400];
source_data.ybox     = [-0.1   5]*1000;
 
%==================================================================
%  
%  Define surface data:
%  
%==================================================================

% This is a homogeneous flat surface, with vacuum over top

surface_data.type  = 'V' ;
surface_data.ptype = 'H' ;
surface_data.itype = 'FL';
surface_data.x     = [x1 x2];
surface_data.y     = [-6   6]*1000;
surface_data.z     = [0 0;0 0];
surface_data.units = 'W';
surface_data.properties = [0 0 0 0 0];

%==================================================================
%  
%  Define sound speed data:
%  
%==================================================================

c0 = 1488.2; % Page 113

ssp_data.ctype = 'ISOV';
ssp_data.x   = [];
ssp_data.y   = [];
ssp_data.z   = [0   z2]';
ssp_data.c   = [c0  c0]';

%==================================================================
%  
%  Define object data:
%  
%==================================================================

object_data.nobjects = 0; % Number of objects

%==================================================================
%  
%  Define bottom data:
%  
%==================================================================

xbty = [x1  x2];
ybty = [-6   6]*1000;
zbty = [z1 z2;z1 z2];

bottom_data.type  = 'E' ;
bottom_data.ptype = 'H' ;
bottom_data.itype = '2P';
bottom_data.x     = xbty;
bottom_data.y     = ybty;
bottom_data.z     = zbty;
bottom_data.units = 'W';
bottom_data.properties = [1700 0 1.99 0.5 0];

%==================================================================
%  
%  Define array data:
%  
%==================================================================

yarray = 0;
xarray = 0;
zarray = 0; 

output_data.x	= xarray;
output_data.y	= yarray;
output_data.z	= zarray;
output_data.nxa = 1;
output_data.nya = 1;
output_data.nza = 1;

%==================================================================
%  
%  Define output data:
%  
%==================================================================

output_data.ctype = 'RCO';
output_data.miss  = 1;

%==================================================================
%  
%  Call the function:
%  
%==================================================================

disp('Writing TRACEO3D waveguide input file...')

wtraceo3dinfil('lmacnrs.in',case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data);

disp('Calling TRACEO3D...')

system('traceo3d.exe lmacnrs.in');

load rco

rayname = 'ray00000';

for i = 1:nphi*nthetas

    if i < 10
    
    myexpr =  [ 'rayxyz = ray0000' num2str(i) ';'];
    
    elseif i < 100
     
     myexpr = [ 'rayxyz = ray000' num2str(i) ';'];
     
    elseif i < 1000
    
    myexpr = [ 'rayxyz = ray00' num2str(i) ';'];
    
    else
    
    end 
    
    eval( myexpr )
    
    x = rayxyz(1,:);
    y = rayxyz(2,:);
    z = rayxyz(3,:);
    
    plot3(x,y,-z,'k'), hold on
    
end

mesh([x1 x2],[0 5000],-zbty)
plot3(8.3*ones(1,20),linspace(500,5000,20),-10*ones(1,20),'ko','MarkerFaceColor',[1 1 1],'MarkerSize',10)
plot3(0,0,-8.3,'k*','MarkerFaceColor',[0 0 0],'MarkerSize',15)
plot3(0,0,-8.3,'ko','MarkerSize',16)
%axis([x1 x2 0 5000 -max([z1 z2]) 0])
view(116,90)
ylim([0 5000])
zlim([-zmax 0])
xlabel('X')
ylabel('Y')
zlabel('Z')

hold off

disp('done.')
