%==================================================================
%  
%  TRACEO3D: LMA CNRS (Frederic) Experimental Data
%  Faro, Ter Jun 13 12:41:49 WEST 2017
%  Written by Tordar 
%  
%==================================================================

clear all, close all

!rm -rf cpr.mat

disp('LMA CNRS Tank Experiment:') 

case_title = 'LMA_CNRS_Tank_Experiment';

%==================================================================
%  
%  Define source data:
%  
%==================================================================

freq = 150; ray_step = 1;

slope = 4.55;
k = tan( slope*pi/180 ); z0 = 44.4;
x1 = -z0/k;
x2 =   -x1;
z1 = k*x1 + z0;
z2 = k*x2 + z0;

zs = 8.3;

xs = [0 0 zs];

thetamax = 45;
nthetas = 390;
thetas = linspace(-thetamax,thetamax,nthetas);

dphi = 20;
phi = [90:0.1:90+dphi]; nphi = length( phi );

source_data.ds       = ray_step;
source_data.position = xs;
source_data.f	     = freq;
source_data.thetas   = thetas;
source_data.phi      = phi;
source_data.nthetas   = nthetas;
source_data.nphi      = nphi;
source_data.xbox     = [-500 500];
source_data.ybox     = [-5     5]*1000;
 
%==================================================================
%  
%  Define surface data:
%  
%==================================================================

% This is a homogeneous flat surface, with vacuum over top

surface_data.type  = 'V' ;
surface_data.ptype = 'H' ;
surface_data.itype = 'FL';
surface_data.x     = [x1 x2];
surface_data.y     = [-6   6]*1000;
surface_data.z     = [0 0;0 0];
surface_data.units = 'W';
surface_data.properties = [0 0 0 0 0];

%==================================================================
%  
%  Define sound speed data:
%  
%==================================================================

c0 = 1488.2; % Page 113

ssp_data.ctype = 'ISOV';
ssp_data.x   = [];
ssp_data.y   = [];
ssp_data.z   = [0   z2]';
ssp_data.c   = [c0  c0]';

%==================================================================
%  
%  Define object data:
%  
%==================================================================

object_data.nobjects = 0; % Number of objects

%==================================================================
%  
%  Define bottom data:
%  
%==================================================================

xbty = [x1  x2];
ybty = [-6   6]*1000;
zbty = [z1 z2;z1 z2];

bottom_data.type  = 'E' ;
bottom_data.ptype = 'H' ;
bottom_data.itype = '2P';
bottom_data.x     = xbty;
bottom_data.y     = ybty;
bottom_data.z     = zbty;
bottom_data.units = 'W';
bottom_data.properties = [1660 0 1.99 0.5 0];

%==================================================================
%  
%  Define array data:
%  
%==================================================================

yarray = [0.035:0.005:4.95]*1000; nya = length( yarray );
xarray = zeros( size( yarray ) );
zarray =  ones( size( yarray ) )*10; 

output_data.x	= xarray;
output_data.y	= yarray;
output_data.z	= zarray;
output_data.nxa = 0;
output_data.nya = nya;
output_data.nza = 0;

%==================================================================
%  
%  Define output data:
%  
%==================================================================

output_data.ctype = 'CPR';
output_data.miss  = 1;

%==================================================================
%  
%  Call the function:
%  
%==================================================================

disp('Writing TRACEO3D waveguide input file...')

wtraceo3dinfil('lmacnrs.in',case_title,source_data,surface_data,ssp_data,object_data,bottom_data,output_data);

disp('Calling TRACEO3D...')

tic
system('traceo3d.exe lmacnrs.in');
toc

load cpr 

tl = 20*log10( abs( pressure ) );

figure
plot(yarray3d/1000, tl,'LineWidth',2)
box on, grid on
xlabel('ACROSS SLOPE RANGE [in m]')
ylabel('TLOSS [in dB]')
title('TRACEO3D LMA CNRS H1 @ 150 Hz')

disp('done.')
